@@ -88,19 +88,27 @@ class BasicNeedsForExtractor:
         return c_fo.fn_get_file_datetime_verdict(
             local_logger, in_dict['file']['name'], 'last modified', datetime.timestamp(r_dt))
 
-    @staticmethod
-    def fn_set_extract_behaviour(in_session):
-        default_value = 'skip-if-output-file-exists'
-        allowed_values = [
-            'skip-if-output-file-exists',
-            'overwrite-if-output-file-exists',
-        ]
-        if ('extract-behaviour' not in in_session) \
-                or (in_session['extract-behaviour'] not in allowed_values):
-            value_to_return = default_value
-        else:
-            value_to_return = in_session['extract-behaviour']
-        return value_to_return
+    def fn_manage_individual_user_settings(self, local_logger, u, current_key):
+        if current_key in u['Storage']:
+            if u['Storage'][current_key] == 'environment variable':  # handling special value
+                local_logger.debug(self.locale.gettext(
+                    'As {current_key} has been set to be an "environment variable"'
+                    + ', value will be read using {environment_variable_name}')
+                                   .replace('{current_key}', current_key)
+                                   .replace('{environment_variable_name}', u[current_key]))
+                env_var_value = os.getenv(u[current_key])
+                if env_var_value is None:
+                    local_logger.error('No such {environment_variable_name} has been found')
+                else:
+                    u[current_key] = env_var_value
+        return u
+
+    def fn_manage_user_settings(self, local_logger, u):
+        if 'Storage' in u:
+            known_account_keys = ['Name', 'Password', 'Username']
+            for current_key in known_account_keys:
+                u = self.fn_manage_individual_user_settings(local_logger, u, current_key)
+        return u
 
     def validate_all_json_files(self, local_logger, timer, extracting_sequences):
         timer.start()
@@ -229,14 +237,14 @@ class BasicNeedsForExtractor:
             item_checked = in_source_systems[in_seq['vdr']][in_seq['typ']]['Server']
             can_proceed_s2 = self.fn_validate_mandatory_properties(
                     in_logger, self.locale.gettext('Source Systems') + ' "' + in_seq['vdr']
-                               + '", "' + in_seq['typ'] + '", "Server" ',
+                    + '", "' + in_seq['typ'] + '", "Server" ',
                     item_checked, [in_seq['grp']])
         can_proceed_s3 = False
         if can_proceed_s2:
             item_checked = in_source_systems[in_seq['vdr']][in_seq['typ']]['Server'][in_seq['grp']]
             can_proceed_s3 = self.fn_validate_mandatory_properties(
                     in_logger, self.locale.gettext('Source Systems') + ' "' + in_seq['vdr']
-                               + '", "' + in_seq['typ'] + '", "' + in_seq['grp'] + '", "Server" ',
+                    + '", "' + in_seq['typ'] + '", "' + in_seq['grp'] + '", "Server" ',
                     item_checked, [in_seq['lyr']])
         return can_proceed_s and can_proceed_s1 and can_proceed_s2 and can_proceed_s3
 
@@ -244,7 +252,8 @@ class BasicNeedsForExtractor:
         is_valid = False
         if in_dc['Source System Properties'] and in_dc['User Secrets']:
             # variable with credentials for source server
-            u = in_user[in_dc['vdr']][in_dc['typ']][in_dc['grp']][in_dc['lyr']][in_dc['ac']]
+            raw_user_settings = in_user[in_dc['vdr']][in_dc['typ']][in_dc['grp']][in_dc['lyr']][in_dc['ac']]
+            u = self.fn_manage_user_settings(local_logger, raw_user_settings)
             is_valid = self.validate_user_secrets(local_logger, u)
             self.connection_details = {
                 'server-vendor-and-type': in_dc['vdtp'],
@@ -258,36 +267,44 @@ class BasicNeedsForExtractor:
         return is_valid
 
     def validate_user_secrets(self, local_logger, in_user_secrets):
-        mandatory_properties = [
-            'Name',
-            'Username',
-            'Password',
-        ]
+        mandatory_properties_dict = {
+            'Name': {
+                'tuple': ['login', 'Your Full Name Here'],
+                'variable': {
+                    'name': 'default_name_value',
+                    'value': in_user_secrets['Name']
+                }
+            },
+            'Username': {
+                'tuple': ['usrnme', 'your_username_goes_here'],
+                'variable': {
+                    'name': 'default_username_value',
+                    'value': in_user_secrets['Username']
+                }
+            },
+            'Password': {
+                'tuple': ['pwd', 'your_password_goes_here'],
+                'variable': {
+                    'name': 'default_password_value',
+                    'value': in_user_secrets['Password']
+                }
+            }
+        }
+        mandatory_properties = list(mandatory_properties_dict.keys())
         is_valid = self.fn_validate_mandatory_properties(local_logger,
                                                          self.locale.gettext('User Secrets')
                                                          + ' ' + self.str_ss,
                                                          in_user_secrets, mandatory_properties)
-        if in_user_secrets['Name'] in ('login', 'Your Full Name Here'):
-            local_logger.warning(
-                    self.locale.gettext('For {str_user_secrets} your "Name" property '
-                                        + 'has the default value: "{default_name_value}" '
-                                        + 'which is unusual')
+        for current_property in mandatory_properties_dict:
+            if in_user_secrets[current_property] in tuple(mandatory_properties_dict[current_property]['tuple']):
+                local_logger.warning(
+                    self.locale.gettext('For {str_user_secrets} your "' + current_property + '" property '
+                                        + 'has the default value: "{'
+                                        + mandatory_properties_dict[current_property]['variable']['name']
+                                        + '}" which is unusual')
                         .replace('{str_user_secrets}', self.str_ss)
-                        .replace('{default_name_value}', in_user_secrets['Name']))
-        if in_user_secrets['Username'] in ('usrnme', 'your_username_goes_here'):
-            local_logger.warning(
-                    self.locale.gettext('For {str_user_secrets} your "Username" property '
-                                        + 'has the default value: "{default_username_value}" '
-                                        + 'which is unusual')
-                        .replace('{str_user_secrets}', self.str_ss)
-                        .replace('{default_username_value}', in_user_secrets['Username']))
-        if in_user_secrets['Password'] in ('pwd', 'your_password_goes_here'):
-            local_logger.warning(
-                    self.locale.gettext('For {str_user_secrets} your "Password" property '
-                                        + 'has the default value: "{default_password_value}" '
-                                        + 'which is unusual')
-                        .replace('{str_user_secrets}', self.str_ss)
-                        .replace('{default_password_value}', in_user_secrets['Password']))
+                        .replace(mandatory_properties_dict[current_property]['variable']['name']
+                                 , mandatory_properties_dict[current_property]['variable']['value']))
         return is_valid
 
     def validate_user_secrets_file(self, local_logger, in_seq, in_user_secrets):
@@ -305,14 +322,13 @@ class BasicNeedsForExtractor:
             item_checked = in_user_secrets[in_seq['vdr']][in_seq['typ']]
             can_proceed_u2 = self.fn_validate_mandatory_properties(
                     local_logger, self.locale.gettext('User Secrets') + ' "' + in_seq['vdr']
-                                  + '", "' + in_seq['typ'] + '", "Server" ',
+                    + '", "' + in_seq['typ'] + '", "Server" ',
                     item_checked, [in_seq['grp']])
         can_proceed_u3 = False
         if can_proceed_u2:
             item_checked = in_user_secrets[in_seq['vdr']][in_seq['typ']][in_seq['grp']]
             can_proceed_u3 = self.fn_validate_mandatory_properties(
                     local_logger, self.locale.gettext('User Secrets') + ' "' + in_seq['vdr']
-                                  + '", "' + in_seq['typ'] + '", "'
-                                  + in_seq['grp'] + '", "Server" ',
+                    + '", "' + in_seq['typ'] + '", "' + in_seq['grp'] + '", "Server" ',
                     item_checked, [in_seq['lyr']])
         return can_proceed_u and can_proceed_u1 and can_proceed_u2 and can_proceed_u3
