@@ -9,6 +9,8 @@ from datetime import datetime, timedelta
 import datedelta
 # package to add support for multi-language (i18n)
 import gettext
+# package to perform mathematical operations
+import math
 # package to handle files/folders and related metadata/operations
 import os
 # package regular expressions
@@ -18,6 +20,14 @@ import re
 class ParameterHandling:
     known_expressions = {
         'year': ['CY', 'CurrentYear'],
+        'semester': ['CYCS', 'CurrentYearCurrentSemester'],
+        'semester_week': ['CYCSCW', 'CurrentYearCurrentSemesterCurrentWeek'],
+        'just_semester': ['CS', 'CurrentSemester'],
+        'quarter': ['CYCQ', 'CurrentYearCurrentQuarter'],
+        'quarter_week': ['CYCQCW', 'CurrentYearCurrentQuarterCurrentWeek'],
+        'just_quarter': ['CQ', 'CurrentQuarter'],
+        'fiscal_period': ['CYCFP', 'CurrentYearCurrentFiscalPeriod'],
+        'just_fiscal_period': ['CFP', 'FiscalPeriod'],
         'month': ['CYCM', 'CurrentYearCurrentMonth'],
         'just_month': ['CM', 'CurrentMonth'],
         'week': ['CYCW', 'CurrentYearCurrentWeek'],
@@ -28,6 +38,8 @@ class ParameterHandling:
     }
     output_standard_formats = {
         'year': '%Y',
+        'fiscal_period': '%Y0%m',
+        'just_fiscal_period': '0%m',
         'month': '%Y%m',
         'just_month': '%m',
         'day': '%Y%m%d',
@@ -37,9 +49,9 @@ class ParameterHandling:
     locale = None
 
     def __init__(self, in_language='en_US'):
-        file_parts = os.path.normpath(os.path.abspath(__file__)).replace('\\', os.path.altsep)\
+        file_parts = os.path.normpath(os.path.abspath(__file__)).replace('\\', os.path.altsep) \
             .split(os.path.altsep)
-        locale_domain = file_parts[(len(file_parts)-1)].replace('.py', '')
+        locale_domain = file_parts[(len(file_parts) - 1)].replace('.py', '')
         locale_folder = os.path.normpath(os.path.join(
             os.path.join(os.path.altsep.join(file_parts[:-2]), 'project_locale'), locale_domain))
         self.locale = gettext.translation(locale_domain, localedir=locale_folder,
@@ -74,6 +86,11 @@ class ParameterHandling:
             expression_parts[2] = 0
         final_dates = {
             'year': in_date + datedelta.datedelta(years=int(expression_parts[2])),
+            'semester': in_date + datedelta.datedelta(months=int(expression_parts[2]) * 6),
+            'semester_week': in_date + timedelta(weeks=int(expression_parts[2])),
+            'quarter': in_date + datedelta.datedelta(months=int(expression_parts[2]) * 3),
+            'quarter_week': in_date + timedelta(weeks=int(expression_parts[2])),
+            'fiscal_period': in_date + datedelta.datedelta(months=int(expression_parts[2])),
             'month': in_date + datedelta.datedelta(months=int(expression_parts[2])),
             'week': in_date + timedelta(weeks=int(expression_parts[2])),
             'day': in_date + timedelta(days=int(expression_parts[2])),
@@ -90,7 +107,7 @@ class ParameterHandling:
                 'A known expression "{expression_parts}" has to be interpreted')
                                .replace('{expression_parts}', str_expression))
             final_string = self.interpret_known_expression(
-                    datetime.now(), expression_parts, in_start_iso_weekday)
+                datetime.now(), expression_parts, in_start_iso_weekday)
             local_logger.debug(self.locale.gettext(
                 'Known expression "{expression_parts}" has been interpreted as {final_string}')
                                .replace('{expression_parts}', str_expression)
@@ -111,7 +128,7 @@ class ParameterHandling:
         if reg_ex:
             parameter_value_parts = reg_ex.group().split('_')
             calculated = self.calculate_date_from_expression(
-                    local_logger, parameter_value_parts, in_start_iso_weekday)
+                local_logger, parameter_value_parts, in_start_iso_weekday)
             value_to_return = re.sub(reg_ex.group(), calculated, crt_parameter)
             local_logger.debug(self.locale.gettext(
                 'Current Parameter is STR and has been re-interpreted as value: "{str_value}"')
@@ -148,8 +165,8 @@ class ParameterHandling:
             if 'parameters-handling-rules' in given_session:
                 parameter_rules = given_session['parameters-handling-rules']
             tp = self.build_parameters(
-                    local_logger, given_session['parameters'], parameter_rules,
-                    in_start_iso_weekday)
+                local_logger, given_session['parameters'], parameter_rules,
+                in_start_iso_weekday)
         return tp
 
     def interpret_known_expression(self, ref_date, expression_parts, in_start_iso_weekday):
@@ -159,31 +176,58 @@ class ParameterHandling:
         finalized_date = ref_date
         if len(expression_parts) > 2:
             finalized_date = self.calculate_date_deviation(ref_date, deviation, expression_parts)
-        year_number_string, week_number_string = self.get_week_number_as_two_digits_string(
+        if deviation_original in ('just_semester', 'just_quarter', 'just_week',
+                                  'semester', 'semester_week', 'quarter', 'quarter_week', 'week'):
+            year_number_string, week_number_string = self.get_week_number_as_two_digits_string(
                 finalized_date, in_start_iso_weekday)
-        if deviation_original == 'week':
-            final_string = year_number_string + week_number_string
-        elif deviation_original == 'just_week':
-            final_string = week_number_string
+            # to ensure less than 4 days within last week of year r reported to next year (ISO)
+            semester_year_string = str(math.ceil(int(week_number_string) / 26))
+            quarter_string = str(min(int(week_number_string),
+                                     math.ceil(int(datetime.strftime(finalized_date, '%m')) / 3)))
+            # values pre-calculated
+            values_to_pick = {
+                'just_semester': 'H' + semester_year_string,
+                'just_quarter': 'Q' + quarter_string,
+                'just_week': week_number_string,
+                'semester': str(datetime.strftime(finalized_date, '%Y'))
+                            + 'H' + semester_year_string,
+                'semester_week': year_number_string + 'H' + semester_year_string
+                                 + 'wk' + week_number_string,
+                'quarter': str(datetime.strftime(finalized_date, '%Y')) + 'Q' + quarter_string,
+                'quarter_week': year_number_string + 'Q' + quarter_string
+                                + 'wk' + week_number_string,
+                'week': year_number_string + week_number_string,
+            }
+            final_string = values_to_pick.get(deviation_original)
         else:
             final_string = datetime.strftime(
-                    finalized_date, self.output_standard_formats.get(deviation_original))
+                finalized_date, self.output_standard_formats.get(deviation_original))
         return final_string
 
-    def manage_parameter_value(self, local_logger, given_prefix, given_parameter,
-                               given_parameter_rules):
+    def manage_parameter_value(self, in_logger, in_prefix, in_parameter, in_parameter_rules):
         element_to_join = ''
-        if given_prefix == 'dict':
-            element_to_join = given_parameter.values()
-        elif given_prefix == 'list':
-            element_to_join = given_parameter
-        local_logger.debug(self.locale.gettext(
+        if in_prefix == 'dict':
+            element_to_join = in_parameter.values()
+        elif in_prefix == 'list':
+            element_to_join = in_parameter
+        in_logger.debug(self.locale.gettext(
             'Current Parameter is {parameter_type} and has the value: "{str_value}"')
-                           .replace('{parameter_type}', self.locale.gettext(given_prefix.upper()))
-                           .replace('{str_value}', str(element_to_join)))
-        return given_parameter_rules[given_prefix + '-values-prefix'] \
-            + given_parameter_rules[given_prefix + '-values-glue'].join(element_to_join) \
-            + given_parameter_rules[given_prefix + '-values-suffix']
+                        .replace('{parameter_type}', self.locale.gettext(in_prefix.upper()))
+                        .replace('{str_value}', str(element_to_join)))
+        return in_parameter_rules[in_prefix + '-values-prefix'] \
+               + in_parameter_rules[in_prefix + '-values-glue'].join(element_to_join) \
+               + in_parameter_rules[in_prefix + '-values-suffix']
+
+    @staticmethod
+    def set_default_starting_weekday(in_dict):
+        week_starts_with_iso_weekday = 1
+        if 'start-iso-weekday' in in_dict['session']:
+            if in_dict['session']['start-iso-weekday'] == 'inherit-from-parent':
+                in_dict['session']['start-iso-weekday'] = in_dict['query']['start-iso-weekday']
+            elif in_dict['session']['start-iso-weekday'] == 'inherit-from-grand-parent':
+                in_dict['session']['start-iso-weekday'] = in_dict['sequence']['start-iso-weekday']
+            week_starts_with_iso_weekday = in_dict['session']['start-iso-weekday']
+        return week_starts_with_iso_weekday
 
     def simulate_final_query(self, local_logger, timer, in_query, in_parameters_number, in_tp):
         timer.start()
@@ -214,11 +258,11 @@ class ParameterHandling:
                                    .replace('{parameter_type}', self.locale.gettext('STR'))
                                    .replace('{str_value}', crt_parameter))
                 working_list[ndx] = self.eval_expression(
-                        local_logger, crt_parameter, in_start_iso_weekday)
+                    local_logger, crt_parameter, in_start_iso_weekday)
             elif current_parameter_type in (list, dict):
                 prefix = str(current_parameter_type).replace("<class '", '').replace("'>", '')
                 working_list[ndx] = self.manage_parameter_value(
-                        local_logger, prefix.lower(), crt_parameter, given_parameter_rules)
+                    local_logger, prefix.lower(), crt_parameter, given_parameter_rules)
         final_tuple = tuple(working_list)
         local_logger.debug(self.locale.gettext('Final Tuple for Parameters is: {final_tuple}')
                            .replace('{final_tuple}', str(final_tuple)))
